!---------------------------------------------------------------------
!     Copyright (C) GFD Dennou Club, 2004. All rights reserved.
!---------------------------------------------------------------------
! histget.f90 - Sample program for gt4_history/gt4f90io.
!               Test Program for "HistoryGet".
!
! History
!   2004/03/31 Morikawa Yasuhiro     create
!   2004/08/05 Morikawa Yasuhiro     add "single & double precision check"
!   2004/08/08 Morikawa Yasuhiro     change to pointer array test

program histget
    use gt4f90io
    use dc_trace,   only: SetDebug, DataDump
    use dc_message, only: MessageNotify
    use dc_types,   only: string
    use gt4_history
implicit none
    integer            :: i,j,k,l                        ! ƕϐ
    integer, parameter :: nx=10, ny=20, nz=30            ! Obh
    real,    parameter :: x(nx)=(/(1.0*(i-1),i=1,nx)/)   ! xWϐ
    real(8), parameter :: y(ny)=(/(10.0*(i-1),i=1,ny)/)  ! yWϐ
    real(8), parameter :: z(nz)=(/(100.0*(i-1),i=1,nz)/) ! zWϐ

    real   , target    :: u           ! o͗pz
    real(8), target    :: v(nx)       ! o͗p 1 z
    real(8), target    :: v2(nz)      ! o͗p 1 z񂻂2
    real   , target    :: w(nx,ny)    ! o͗p 2 z
    real(8), target    :: q(nx,ny,nz) ! o͗p 3 z

    real   , pointer   :: up          ! ͗pz|C^
    real(8), pointer   :: vp(:)       ! ͗p 1 z|C^
    real   , pointer   :: wp(:,:)     ! ͗p 2 z|C^
    real(8), pointer   :: qp(:,:,:)   ! ͗p 3 z|C^

    real   , pointer   :: xp(:)       ! ͗p xWz|C^
    real(8), pointer   :: yp(:)       ! ͗p yWz|C^
    real(8), pointer   :: zp(:)       ! ͗p zWz|C^

    character(string), parameter  :: subname = 'histget'
continue

    call SetDebug

    !-----------------------------------------------------------------
    ! ܂͓͗p̃t@C쐬
    !-----------------------------------------------------------------

    call HistoryCreate(file='xhistget/xhistget1.nc', &
        & title='gt4_history HistoryGet test 1', &
        & source='gt4f90io/Fortran library test kit', &
        & institution='GFD Dennou Club', &
        & dims=(/'x','y','z','t'/), &
        & dimsizes=(/10,20,30,0/), &
        & longnames=(/'X-coordinate','Y-coordinate', &
                   &  'Z-coordinate','time        '/), &
        & units=(/'m','m','m','s'/), &
        & origin=0.0, interval=0.2, &
        & xtypes=(/'real  ','double','double','real  '/))

    call HistoryPut('x',x)   ! ϐo
    call HistoryPut('y',y)   ! ϐo
    call HistoryPut('z',z)   ! ϐo

    call HistoryAddVariable('u', dims=(/'t'/), &
        & longname='Non Dimensional any quantity', &
        & units='non-dimensional')
    call HistoryAddVariable('v', dims=(/'x','t'/), &
        & longname='1 Dimensional any quantity', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('v2', dims=(/'z','t'/), &
        & longname='1 Dimensional any quantity II', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('w', dims=(/'x','y','t'/), &
        & longname='2 Dimensional any quantity', &
        & units='2-dimensional any unit')
    call HistoryAddVariable('q', dims=(/'x','y','z','t'/), &
        & longname='3 Dimensional any quantity', &
        & units='3-dimensional any unit', xtype='double')

    do, l = 1, 24
       u = real(l)*0.100000001
        do i = 1, 10
           v(i) = dble(l)*0.100000001d0+dble(i)*10.0000001d0
           do j = 1, 20
              w(i,j) = real(l)*0.100000001 + real(i)*10.0000001 &
                       & + real(j)*1000.00001
              do k = 1, 30
                 q(i,j,k) = dble(l)*0.100000001d0     &
                            & + dble(i)*10.0000001d0  &
                            & + dble(j)*1000.00001d0  &
                            & + dble(k)*100000.001d0

                 v2(k) = dble(k)*100000.001d0
              enddo
           enddo
        enddo
        call HistoryPut('t', real(l)*0.2)
        call HistoryPut('u', u)
        call HistoryPut('v', v)
        call HistoryPut('v2', v2)
        call HistoryPut('w', w)
        call HistoryPut('q', q)

    enddo

    call DataDump('v', v, strlen=10)
    call DataDump('w', dble(w), strlen=60)
    call DataDump('q(1,1,', q(1,1,:))

    call HistoryClose

    call MessageNotify('M', subname, 'Input file <%c> is generated.', &
         & c1='xhistget/xhistget1.nc')

    !-----------------------------------------------------------------
    ! Œ蒷zɂ̓eXg (FXȍ@œ)
    !-----------------------------------------------------------------

    call MessageNotify('M', subname, 'Getting by Fixed-Length Array.')

    call HistoryCreate(file='xhistget/xhistget2.nc', &
        & title='gt4_history HistoryGet test 1', &
        & source='gt4f90io/Fortran library test kit', &
        & institution='GFD Dennou Club', &
        & dims=(/'x','y','z','t'/), &
        & dimsizes=(/10,20,30,0/), &
        & longnames=(/'X-coordinate','Y-coordinate', &
                   &  'Z-coordinate','time        '/), &
        & units=(/'m','m','m','s'/), &
        & origin=0.0, interval=0.2, &
        & xtypes=(/'real  ','double','double','real  '/))

    call HistoryPut('x',x)   ! ϐo
    call HistoryPut('y',y)   ! ϐo
    call HistoryPut('z',z)   ! ϐo

    call HistoryAddVariable('u', dims=(/'t'/), &
        & longname='Non Dimensional any quantity', &
        & units='non-dimensional')
    call HistoryAddVariable('v', dims=(/'x','t'/), &
        & longname='1 Dimensional any quantity', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('v2', dims=(/'z','t'/), &
        & longname='1 Dimensional any quantity II', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('w', dims=(/'x','y','t'/), &
        & longname='2 Dimensional any quantity', &
        & units='2-dimensional any unit')
    call HistoryAddVariable('q', dims=(/'x','y','z','t'/), &
        & longname='3 Dimensional any quantity', &
        & units='3-dimensional any unit', xtype='double')

    do, l = 1, 23
       call HistoryPut('t', real(l)*0.2)
       call HistoryGet(file='xhistget/xhistget1.nc', varname='u', & 
            &          array=u, time=real(l)*0.2)
       call HistoryPut('u', u)
       call HistoryGet('xhistget/xhistget1.nc', 'v', v, dble(l)*0.2d0)
       call HistoryPut('v', v)
       call HistoryGet('xhistget/xhistget1.nc', 'v2', v2, time='^'//toChar(l))
       call HistoryPut('v2', v2)
       call HistoryGet('xhistget/xhistget1.nc', 'w', w, time='^'//toChar(l))
       call HistoryPut('w', w)
       call HistoryGet('xhistget/xhistget1.nc', 'q', q, '^'//toChar(l))
       call HistoryPut('q', q)
    enddo

    ! Ԃ̎ʂ̃eXg
    l = 24
    call HistoryPut('t', real(l)*0.2)
    call HistoryGet(file='xhistget/xhistget1.nc', varname='u', array=u)
    call HistoryPut('u', u)
    call HistoryGet('xhistget/xhistget1.nc', 'v', v)
    call HistoryPut('v', v)
    call HistoryGet('xhistget/xhistget1.nc', 'v2', v2, time='^'//toChar(l))
    call HistoryPut('v2', v2)
    call HistoryGet('xhistget/xhistget1.nc', 'w', w)
    call HistoryPut('w', w)
    call HistoryGet('xhistget/xhistget1.nc', 'q', q, '^'//toChar(l))
    call HistoryPut('q', q)

    call HistoryClose


    !-----------------------------------------------------------------
    !   |C^zɂ̓eXg (FXȍ@œ)
    !-----------------------------------------------------------------
    call MessageNotify('M', subname, 'Getting by Pointer Array.')

    call HistoryCreate(file='xhistget/xhistget3.nc', &
        & title='gt4_history HistoryGet test 1', &
        & source='gt4f90io/Fortran library test kit', &
        & institution='GFD Dennou Club', &
        & dims=(/'x','y','z','t'/), &
        & dimsizes=(/10,20,30,0/), &
        & longnames=(/'X-coordinate','Y-coordinate', &
                   &  'Z-coordinate','time        '/), &
        & units=(/'m','m','m','s'/), &
        & origin=0.0, interval=0.2, &
        & xtypes=(/'real  ','double','double','real  '/))

    call HistoryPut('x',x)   ! ϐo
    call HistoryPut('y',y)   ! ϐo
    call HistoryPut('z',z)   ! ϐo

    call HistoryAddVariable('u', dims=(/'t'/), &
        & longname='Non Dimensional any quantity', &
        & units='non-dimensional')
    call HistoryAddVariable('v', dims=(/'x','t'/), &
        & longname='1 Dimensional any quantity', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('v2', dims=(/'z','t'/), &
        & longname='1 Dimensional any quantity II', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('w', dims=(/'x','y','t'/), &
        & longname='2 Dimensional any quantity', &
        & units='2-dimensional any unit')
    call HistoryAddVariable('q', dims=(/'x','y','z','t'/), &
        & longname='3 Dimensional any quantity', &
        & units='3-dimensional any unit', xtype='double')

    do, l = 1, 23
       call HistoryPut('t', real(l)*0.2)
       call HistoryGetPointer(file='xhistget/xhistget1.nc', varname='u', & 
            &          array=up, time=real(l)*0.2)
       call HistoryPut('u', up)
       ! rŕʂ̊֐ɊtĂIɉ邩eXg
       up => u
       call HistoryGetPointer('xhistget/xhistget1.nc', 'v', vp, dble(l)*0.2d0)
       call HistoryPut('v', vp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'v2', vp, time='^'//toChar(l))
       call HistoryPut('v2', vp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'w', wp, time='^'//toChar(l))
       call HistoryPut('w', wp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'q', qp, '^'//toChar(l))
       call HistoryPut('q', qp)
    enddo

    ! Ԃ̎ʂ̃eXg
    l = 24
    call HistoryPut('t', real(l)*0.2)
    call HistoryGetPointer(file='xhistget/xhistget1.nc', varname='u', array=up)
    call HistoryPut('u', up)
    call HistoryGetPointer('xhistget/xhistget1.nc', 'v', vp)
    call HistoryPut('v', vp)
    call HistoryGetPointer('xhistget/xhistget1.nc', 'v2', vp, dble(l)*0.2d0)
    call HistoryPut('v2', vp)
    call HistoryGetPointer('xhistget/xhistget1.nc', 'w', wp)
    call HistoryPut('w', wp)
    call HistoryGetPointer('xhistget/xhistget1.nc', 'q', qp, '^'//toChar(l))
    call HistoryPut('q', qp)

    call HistoryClose


    !-----------------------------------------------------------------
    !   f[^̓̓eXg (FXȍ@œ)
    !-----------------------------------------------------------------
    call MessageNotify('M', subname, 'Getting by Various Forms.')

    call HistoryCreate(file='xhistget/xhistget4.nc', &
        & title='gt4_history HistoryGet test 1', &
        & source='gt4f90io/Fortran library test kit', &
        & institution='GFD Dennou Club', &
        & dims=(/'x','y','z','t'/), &
        & dimsizes=(/10,20,30,0/), &
        & longnames=(/'X-coordinate','Y-coordinate', &
                   &  'Z-coordinate','time        '/), &
        & units=(/'m','m','m','s'/), &
        & origin=0.0, interval=0.2, &
        & xtypes=(/'real  ','double','double','real  '/))

    ! f[^͂
    call HistoryGetPointer('xhistget/xhistget1.nc', 'x' ,xp)   ! x 
    call HistoryGetPointer('xhistget/xhistget1.nc', 'y' ,yp)   ! y 
    call HistoryGetPointer(file='xhistget/xhistget1.nc', varname='z' ,array=zp)   ! z 

    call HistoryPut('x',xp)   ! ϐo
    call HistoryPut('y',yp)   ! ϐo
    call HistoryPut('z',zp)   ! ϐo

    call HistoryAddVariable('u', dims=(/'t'/), &
        & longname='Non Dimensional any quantity', &
        & units='non-dimensional')
    call HistoryAddVariable('v', dims=(/'x','t'/), &
        & longname='1 Dimensional any quantity', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('v2', dims=(/'z','t'/), &
        & longname='1 Dimensional any quantity II', &
        & units='1-dimensional any unit', xtype='double')
    call HistoryAddVariable('w', dims=(/'x','y','t'/), &
        & longname='2 Dimensional any quantity', &
        & units='2-dimensional any unit')
    call HistoryAddVariable('q', dims=(/'x','y','z','t'/), &
        & longname='3 Dimensional any quantity', &
        & units='3-dimensional any unit', xtype='double')

    do, l = 1, 24
       call HistoryPut('t', real(l)*0.2)
       call HistoryGetPointer(file='xhistget/xhistget1.nc', varname='u', & 
            &          array=up, time=real(l)*0.2)
       call HistoryPut('u', up)
       ! rŕʂ̊֐ɊtĂIɉ邩eXg
       up => u
       call HistoryGetPointer('xhistget/xhistget1.nc', 'v', vp, dble(l)*0.2d0)
       call HistoryPut('v', vp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'v2', vp, time='^'//toChar(l))
       call HistoryPut('v2', vp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'w', wp, time='^'//toChar(l))
       call HistoryPut('w', wp)
       call HistoryGetPointer('xhistget/xhistget1.nc', 'q', qp, '^'//toChar(l))
       call HistoryPut('q', qp)
    enddo

    call HistoryClose

end program
